/**
 * Created by Administrator on 2014/12/16.
 * 本地储存和读取数据
 * 调用方法：存数据storage('name','value')
 *           读数据storage('name')
 *           清除单个数据/清空所有数据storage.reset('name')/storage.reset()
 *           判断是否支持本地储存storage()
 *           判断是否支持cookie/sessionStorage：storage.cookie()/storage.session();
 *           指定使用cookie/sessionStorage存取: storage.cookie('name'[,'value'])/storage.session('name'[,'value'])
 * 返回值: 存取返回存取的value，判断返回true/false
 */
define(function(){
    var oReturn;
    /*不支持sessionStorage的浏览器使用cookie，定义cookie存取*/
    /*把模块写成一个对象，所有模块成员都放在这个对象里面*/
    var fCookie = function (c_name,c_value) {
        if(c_value === undefined && c_name === undefined){
            return navigator.cookieEnabled;
        }
        else if(c_value === undefined){
            if (document.cookie.length > 0) {     /*先查询cookie是否为空，为空就return undefined*/
                var c_start = document.cookie.indexOf(c_name + "=");
                if (c_start != -1) {
                    c_start = c_start + c_name.length + 1;
                    var c_end = document.cookie.indexOf(";", c_start);
                    if (c_end == -1)
                        c_end = document.cookie.length;
                    return decodeURI(document.cookie.substring(c_start, c_end));
                }
            }
            return undefined;
        }
        else {
            document.cookie = c_name + '=' + encodeURI(c_value);
            return c_value;
        }
    };
    /*删除cookie*/
    fCookie.reset = function (c_name)
    {
        var names;
        if(c_name == undefined){
            names = document.cookie.match(/[^ =;]+(?=\=)/g);
        }
        else names = [c_name];
        names.forEach(function(name){
            var exp = new Date();
            exp.setTime(exp.getTime() - 1);
            var cval = fCookie(name);
            if (cval != null) document.cookie = name + "=" + cval + ";expires=" + exp.toGMTString();
        });
    };

    /*定义sessionStorage存取，无参数则判断是否支持sessionStorage*/
    var fSessionItem = function(a,b){
        if(b === undefined && a === undefined){
            return !!sessionStorage;
        }
        else if(b === undefined){
            return sessionStorage.getItem(a);
        }
        else {
            sessionStorage.setItem(a,b);
            return b;
        }
    };
    fSessionItem.reset = function (name){
        if(name == undefined) {
            sessionStorage.clear();
        }
        else {
            sessionStorage.removeItem(name);
        }
    };

    /*定义localStorage存取，无参数则判断是否支持localStorage*/
    var fLocalItem = function(a,b){
        if(b === undefined && a === undefined){
            return !!localStorage;
        }
        else if(b === undefined){
            return localStorage.getItem(a);
        }
        else {
            localStorage.setItem(a,b);
            return b;
        }
    };
    fLocalItem.reset = function (name){
        if(name == undefined) {
            localStorage.clear();
        }
        else {
            localStorage.removeItem(name);
        }
    };

    /*判断浏览器是否支持sessionStorage*/
    if(fSessionItem()){
        oReturn = fSessionItem;
    }
    else if(fCookie()) {
        oReturn = fCookie;
    }
    else {
    /*容错，防止JS出错停止运行*/
        oReturn = function (a,b) {
            console.log('!!!----storage.js 文件错误');
            return undefined;
        };
        oReturn.reset = function (a){
            console.log('!!!----storage.js 文件错误');
            return undefined;
        }
    }

    /*定义模块的其他属性*/
    oReturn.session = fSessionItem;
    oReturn.local = fLocalItem;
    oReturn.cookie = fCookie;

    return oReturn;
});